EPiC = LibStub("AceAddon-3.0"):NewAddon("EPiC", "AceConsole-3.0", "AceEvent-3.0" );
EPiCGUI = LibStub("AceGUI-3.0");
local mainFrame = nil;

local uninitialized = "";
local unset = "unset";
local charName = UnitName("player");
local STATUS_READY="Ready";
local STATUS_ONCOOLDOWN="OnCooldown";
local debugging=false;
local refreshCooldownsComplete=false;

local FILTER_NONE="None";
local FILTER_TOON="Toon";
local FILTER_STATUS="Status";

local LOG_SUPPRESS="suppress";

local passedTests = 0;
local failedTests = 0;

skillWaitingForUpdate=false;
skillWaitingForUpdateName = uninitialized;
skillWaitingForUpdateProfession = uninitialized;

local globalToonDetailKey;

local defaultProfile = {
	realm = {
		toonDetails = {},
		toonsCount = 0,
		windowOrientation = "CENTER",
		windowX = 0.0,
		windowY = 0.0,
	},
	global = {
		staticCooldowns = {
			Alchemy = {
				"Secrets of Draenor Alchemy",
				"Alchemical Catalyst",
			},
			Blacksmithing = {
				"Secrets of Draenor Blacksmithing",
				"Truesteel Ingot",
			},
			Enchanting = {
				"Secrets of Draenor Enchanting",
				"Temporal Crystal",
			},
			Engineering = {
				"Secrets of Draenor Engineering",
				"Gearspring Parts",
			},
			Inscription = {
				"Secrets of Draenor Inscription",
				"War Paints",
			},
			Jewelcrafting = {
				"Secrets of Draenor Jewelcrafting",
				"Taladite Crystal",
			},
			Leatherworking = {
				"Secrets of Draenor Leatherworking",
				"Burnished Leather",
			},
			Tailoring = {
				"Secrets of Draenor Tailoring",
				"Hexweave Cloth",
				"Imperial Silk",
			},
		}
	}
}

function EPiC:GetNewDateTime()
	local newDateTime = {
		year = 0,
		month = 0,
		day = 0,
		hour = 0,
		minute = 0,
	};

	return newDateTime;
end

--function EPiC:GetNewToonDetail()
--	local newToonDetail  = {
--		name = uninitialized,
--		professions = {
--			prof1 = uninitialized,
--			prof2 = uninitialized,
--		},
--		profCooldowns = {},
--	};
--
--	return newToonDetail;
--end

local toonDetail  = {
	name = uninitialized,
	professions = {
		prof1 = uninitialized,
		prof2 = uninitialized,
	},
	profCooldowns = {},
}

function EPiC:SkillUpdate()

	local logHeader = "UpdateSkills()";

	if (skillWaitingForUpdate) then
		local spellName = skillWaitingForUpdateName;
		local professionName = skillWaitingForUpdateProfession;
		
		--Since we've stored relevant details, we can reset the status
		skillWaitingForUpdate = false;
		skillWaitingForUpdateName = uninitialized;
		skillWaitingForUpdateProfession = uninitialized;

		for i=1,GetNumTradeSkills() do
			local skillName, type = GetTradeSkillInfo(i);
			local cooldown = GetTradeSkillCooldown(i);
		
			if (not cooldown) then
				cooldown = 0;
			end
			
			--If actual skill, check for matches otherwise expand headers
			if (skillName and type ~= "header" and type ~= "subheader") then
			
				--Find the cooldown in the profession window
				if(skillName == spellName) then
					EPiC:UpdateProfessionCooldowns(professionName, skillName, cooldown);
					EPiC:Log(logHeader, "Cooldown update successful for " .. skillName);		
					return;
				end
			else 
				ExpandTradeSkillSubClass(i);
			end
		end

		EPiC:Log(logHeader, "Unable to find spell (" .. spellName .. ") in current profession window");
	end
end

function EPiC:SpellCasted(event, ...)
	local logHeader = "SpellCasted()"
		
	if event == "UNIT_SPELLCAST_SUCCEEDED" then
		local unit, spell, _, _, spellID = ...

		if( unit == "player" and EPiC:IsAStaticCooldown(spell)) then

			local professionName = EPiC:GetProfessionFromSkillName(spell);

			EPiC:Log(logHeader, "Waiting for cooldown update for " .. spell);

			skillWaitingForUpdate = true;
			skillWaitingForUpdateName = spell;
			skillWaitingForUpdateProfession = professionName;
		end
	end
end

function EPiC:IsAStaticCooldown(spellName)
	local logHeader = "IsAStaticCooldown()"
	EPiC:Debug(logHeader, spellName);
	
	local prof1CDs = EPiC:GetStaticDailyCooldownsForProfession(EPiC:ToonDetails().professions.prof1);
	local prof2CDs = EPiC:GetStaticDailyCooldownsForProfession(EPiC:ToonDetails().professions.prof2);

	if (EPiC:IsMatchingSkill(prof1CDs, spellName)) then
		EPiC:Debug(logHeader, spellName .. " matches first profession");
		return true;
	end
	
	if (EPiC:IsMatchingSkill(prof2CDs, spellName)) then
		EPiC:Debug(logHeader, spellName .. " matches second profession");
		return true;
	end

	EPiC:Debug(logHeader, spellName .. " doesn't match one of the character's profession");
	return false;
end

function EPiC:ToggleDebugLogging()
	local logHeader = "ToggleDebugLogging()";

	if (debugging) then
		debugging = false;
		EPiC:Log(logHeader, "Debugging is off.");
	else
		debugging = true;
		EPiC:Log(logHeader, "Debugging is on.");
	end
end

function EPiC:OnInitialize()
	
	EPiC:RegisterEvent("TRADE_SKILL_SHOW", "OnProfessionOpen");
	EPiC:RegisterEvent("TRADE_SKILL_UPDATE", "SkillUpdate");
	EPiC:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED", "SpellCasted");

	local AceDB = LibStub:GetLibrary("AceDB-3.0");

	EPiC.db = AceDB:New("EPiC_StaticCooldownDB", defaultProfile);
	globalToonDetailKey = EPiC:GetToonDetailKey(); 

	EPiC:RegisterChatCommand("rl", "EpicReload");
	EPiC:RegisterChatCommand("epicreset", "ReRegisterToon");
	EPiC:RegisterChatCommand("epicsummary", "GetSummaryReport");
	EPiC:RegisterChatCommand("epicdetails", "GetDetailedReport");
	EPiC:RegisterChatCommand("epicshow", "SetupGui");
	EPiC:RegisterChatCommand("epictest", "UnitTest");
	EPiC:RegisterChatCommand("epicupdate", "SkillUpdate");
	EPiC:RegisterChatCommand("epicdebug", "ToggleDebugLogging");

	if (debugging) then EPiC:UnitTest(); end
	
	--EPiC.db:RegisterDefaults(defaultProfile)
	--EPiC.db:ResetDB(defaultProfile)

	self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig");
	self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig");
	self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig");

	self:Print("Initialized");
end

function EPiC:SetOrientation()
	if (mainFrame ~= nil) then

		local orientation, _, _, x, y = mainFrame:GetPoint();

		if (EPiC.db.realm.windowOrientation == nil) then
			EPiC.db.realm.windowOrientation = "LEFT";
		else
			EPiC.db.realm.windowOrientation = orientation;
		end

		if (EPiC.db.realm.windowX == nil) then
			EPiC.db.realm.windowX = 15.09;
		else
			EPiC.db.realm.windowX = x;
		end

		if (EPiC.db.realm.windowY == nil) then
			EPiC.db.realm.windowY = -169.64
		else
			EPiC.db.realm.windowY = y;
		end

		self:Print("Orientation of window has been set")
	end
end

function EPiC:GetCurrentDateTime()
	local _, month, day, year = CalendarGetDate();
	local hour, minute = GetGameTime();

	local curDateTime = EPiC:GetNewDateTime();

	curDateTime.year = year;
	curDateTime.month = month;
	curDateTime.day = day;
	curDateTime.hour = hour;
	curDateTime.minute = minute;

	return curDateTime;
end

function EPiC:GetExpirationDateTime(hour, minute)
	local header = "EPiC:GetExpirationDateTime";
	local expirDateTime = EPiC:GetCurrentDateTime();
	EPiC:Debug(header, EPiC:DateTimeToString(expirDateTime));
	
	expirDateTime.minute = expirDateTime.minute + minute;
	expirDateTime.hour = expirDateTime.hour + hour;
	EPiC:Debug(header, expirDateTime.hour);
	EPiC:Debug(header, expirDateTime.minute);

	if (expirDateTime.minute >= 60) then
		EPiC:Debug(header, "minute over 60");
		expirDateTime.minute = expirDateTime.minute % 60;
		expirDateTime.hour = expirDateTime.hour + 1;
	end

	if (expirDateTime.hour >= 24) then
		EPiC:Debug(header, "hour over 24");
		expirDateTime.hour = expirDateTime.hour % 24;
		expirDateTime.day = expirDateTime.day + 1;
	end

	EPiC:Debug(header, EPiC:DateTimeToString(expirDateTime));
	return expirDateTime;
end

function EPiC:IsOnCooldown(expirationDate)
	local header = "EPiC:IsOnCooldown";
	
	expirYear = expirationDate.year;
	expirMonth = expirationDate.month;
	expirDay = expirationDate.day;
	expirHour = expirationDate.hour;
	expirMinute = expirationDate.minute;

	EPiC:Debug(header, "Expiration1: " .. expirYear .. ":".. expirMonth .. ":" .. expirDay .. ":" .. expirHour .. ":" .. expirMinute);
	local expirDateTime = expirationDate;
	local currentTime = EPiC:GetCurrentDateTime();

	EPiC:Debug(header, "Expiration2: " .. expirYear .. ":".. expirMonth .. ":" .. expirDay .. ":" .. expirHour .. ":" .. expirMinute);
	EPiC:Debug(header, "Current: " .. currentTime.year .. ":" .. currentTime.month .. ":" .. currentTime.day .. ":" .. currentTime.hour .. ":" .. currentTime.minute);

	--Check year expiration
	if		(currentTime.year > expirYear ) then
		EPiC:Debug(header, "Past Date: year");
		return false;
	elseif	(expirYear > currentTime.year) then
		EPiC:Debug(header, "Future Date: year");
		return true;
	end

	if		(currentTime.month > expirMonth ) then
		EPiC:Debug(header, "Past Date: month");
		return false;
	elseif	(expirMonth > currentTime.month) then
		EPiC:Debug(header, "Future Date: month");
		return true;
	end

	if		(currentTime.day > expirDay ) then
		EPiC:Debug(header, "Past Date: day");
		return false;
	elseif	(expirDay > currentTime.day) then
		EPiC:Debug(header, "Future Date: day");
		return true;
	end

	if		(currentTime.hour > expirHour ) then
		EPiC:Debug(header, "Past Date: hour");
		return false;
	elseif	(expirHour > currentTime.hour) then
		EPiC:Debug(header, "Future Date: hour");
		return true;
	end

	if		(currentTime.minute > expirMinute ) then
		EPiC:Debug(header, "Past Date: minute");
		return false;
	elseif	(expirMinute > currentTime.minute) then
		EPiC:Debug(header, "Future Date: minute");
		return true;
	end

	return false;
end

function EPiC:DateTimeToString(dt)
	local dateTimeString = (dt.year .. "/" .. dt.month .. "/" .. dt.day .. " " .. dt.hour .. ":" .. dt.minute);
	return dateTimeString;
end

function EPiC:OnEnable()
	local logHeader = "OnEnable";

	if (EPiC:ToonDetails() == nil) then 
		EPiC:RegisterToon();
	end

	EPiC:Log(logHeader, "Hello " .. EPiC:ToonDetails().name);

	EPiC:RefreshCooldowns();
	EPiC:SetupGui();
end

function EPiC:SetupGui()
	---Start gui setup
	-- Create a container frame
	mainFrame = EPiCGUI:Create("Frame")
	mainFrame:SetCallback("OnClose",
		function(widget) 
			self:Print("Type /epicshow to display UI again");
			EPiCGUI:Release(widget) 
		end
	)
	mainFrame:SetTitle("Elezator's Profession Cooldowns (EPiC)")
	mainFrame:SetStatusText("Version 0.1")
	mainFrame:SetLayout("Flow")
	mainFrame:SetWidth(400)
	mainFrame:SetHeight(300)
	
	if(EPiC.db.realm.windowOrientation ~= nil and EPiC.db.realm.windowX ~= nil and EPiC.db.realm.windowY ~= nil) then
		mainFrame:SetPoint(EPiC.db.realm.windowOrientation, EPiC.db.realm.windowX, EPiC.db.realm.windowY)
	else 
		mainFrame:SetPoint("CENTER", 0.0, 0.0)
	end

	-- Initialize all widgets
	local label = EPiCGUI:Create("Label")
	local orientation = EPiCGUI:Create("Button")
	local dropdown = EPiCGUI:Create("Dropdown")
	local actionButton = EPiCGUI:Create("Button")
	local editBox = EPiCGUI:Create("MultiLineEditBox")

	-- Define refresh button
	orientation:SetRelativeWidth(1.0)
	orientation:SetText("Set Current Location as Default")
	orientation:SetCallback("OnClick", function() 
			EPiC:SetOrientation();
		end
	)
	
	-- Define dropdown
	DROPDOWN_RDY_CDS = "Ready Cooldowns";
	DROPDOWN_SUM_REP = "Summary Report";
	DROPDOWN_DET_REP = "Detailed Report";

	local orderedList = {
		DROPDOWN_RDY_CDS,
		DROPDOWN_SUM_REP,
		DROPDOWN_DET_REP,
	}

	local initialKey = EPiC:GetKeyFromValue(orderedList, DROPDOWN_RDY_CDS)

	dropdown:SetList(orderedList)
	dropdown:SetValue(initialKey) -- initial display
	dropdown:SetRelativeWidth(0.7) -- initial display
	dropdown:SetCallback("OnValueChanged", function(choice) end )

	-- Define edit box
	local timesClicked = 0;
	editBox:SetLabel("Results");
	editBox:SetRelativeWidth(1.0);
	editBox:DisableButton(true);
	--editBox:SetHeight(150);
	editBox:SetNumLines(10);

	-- Define action button
	actionButton:SetRelativeWidth(0.3)
	actionButton:SetText("Go")
	actionButton:SetCallback("OnClick", function() 

		self:Print("Getting choice");
		local choice = orderedList[dropdown:GetValue()];
		self:Print(choice);
		
		local editBoxText = "";
		local editBoxLabel = uninitialized;
		
		if (choice == DROPDOWN_RDY_CDS) then
			editBoxLabel, editBoxText = EPiC:GetSummaryReport("r");
		elseif( choice == DROPDOWN_SUM_REP) then
			editBoxLabel, editBoxText = EPiC:GetSummaryReport();
		elseif( choice == DROPDOWN_DET_REP) then
			editBoxLabel, editBoxText = EPiC:GetDetailedReport();
		end
		
		if (editBoxLabel == uninitialized) then
			editBox:SetLabel("Results");
		else
			editBox:SetLabel("Results: " .. editBoxLabel);
		end

		editBox:SetText(editBoxText);
	end)


	-- Add widgets to main frame
	mainFrame:AddChild(dropdown);
	mainFrame:AddChild(actionButton);
	mainFrame:AddChild(editBox);
	mainFrame:AddChild(orientation);

	-- Override default constructor behavior
	mainFrame:EnableResize(false)

	self:Print("Done initializing frame");
end

function EPiC:OnDisable()
	-- Called when the addon is disabled
end

function EPiC:GetFilters(filter)
	local filterType=FILTER_NONE;
	local filterValue=uninitialized;

	if		(filter == nil)					then filterType=FILTER_NONE; filterValue=uninitialized;
	elseif	(filter == "")					then filterType=FILTER_NONE; filterValue=uninitialized;

	elseif	(filter == STATUS_ONCOOLDOWN)	then filterType=FILTER_STATUS; filterValue=STATUS_ONCOOLDOWN;
	elseif	(filter == "cooldown")			then filterType=FILTER_STATUS; filterValue=STATUS_ONCOOLDOWN;
	elseif	(filter == "Cooldown")			then filterType=FILTER_STATUS; filterValue=STATUS_ONCOOLDOWN;
	elseif	(filter == "cd")				then filterType=FILTER_STATUS; filterValue=STATUS_ONCOOLDOWN;
	elseif	(filter == "CD")				then filterType=FILTER_STATUS; filterValue=STATUS_ONCOOLDOWN;

	elseif (filter == STATUS_READY)			then filterType=FILTER_STATUS; filterValue=STATUS_READY;
	elseif (filter == "ready")				then filterType=FILTER_STATUS; filterValue=STATUS_READY;
	elseif (filter == "Ready")				then filterType=FILTER_STATUS; filterValue=STATUS_READY;
	elseif (filter == "R")					then filterType=FILTER_STATUS; filterValue=STATUS_READY;
	elseif (filter == "r")					then filterType=FILTER_STATUS; filterValue=STATUS_READY;
	elseif (filter == "rdy")				then filterType=FILTER_STATUS; filterValue=STATUS_READY;
	elseif (filter == "Rdy")				then filterType=FILTER_STATUS; filterValue=STATUS_READY;

	else	filterType=FILTER_TOON; filterValue=filter;
	
	end
	
	return filterType, filterValue;
end

function EPiC:GetSummaryReport(input)
	EPiC:RefreshCooldowns(LOG_SUPPRESS);

	local reportText = "";
	local reportName = "";
	local filterType, filterValue = EPiC:GetFilters(input);

	local filterText = "";
	if (filterType ~= FILTER_NONE) then
		filterText = " (Filter=" .. filterValue .. ") ";
	end

	reportName = "Summary Report" .. filterText;

	for toonKey, toonValue in ipairs(EPiC.db.realm.toonDetails) do
		local toonName = toonValue.name;

		if (filterType ~= FILTER_TOON or (filterType == FILTER_TOON and filterValue == toonName)) then
			local professionCooldowns = toonValue.profCooldowns;
			local skillsReadyCount = 0;
			local skillsOnCDCount = 0;
			local skillsTotalCount = 0;

			for key, value in pairs(professionCooldowns) do
				skillsTotalCount = skillsTotalCount + 1;

				if (value.status == STATUS_READY) then
					skillsReadyCount = skillsReadyCount + 1;
				else
					skillsOnCDCount = skillsOnCDCount + 1;
				end
			end
				
			if (filterType == FILTER_STATUS) then
				if (filterValue == STATUS_READY and skillsReadyCount > 0) then
					reportText = EPiC:AppendLineWithPrefix(reportText, toonName, skillsReadyCount .. " Ready; ");
				elseif (filterValue == STATUS_ONCOOLDOWN and skillsOnCDCount > 0) then
					reportText = EPiC:AppendLineWithPrefix(reportText, toonName, skillsOnCDCount .. " OnCooldown; ");
				end
			else
				reportText = EPiC:AppendLineWithPrefix(reportText, toonName, skillsReadyCount .. " Ready; " .. skillsOnCDCount .. " OnCooldown;");
			end
		end
	end

	--reportNamereportText = EPiC:AppendLine(reportText, "--------Summary Report Complete--------");

	return reportName, reportText;
end

function EPiC:GetDetailedReport(input)
	EPiC:RefreshCooldowns(LOG_SUPPRESS);

	local reportText = "";
	local reportName = "Detailed Report";
	--reportText = EPiC:AppendLine(reportText, "--------Detailed Report--------");

	--self:Print("Filters: " .. input);
	--self:Print("");

	for toonKey, toonValue in ipairs(EPiC.db.realm.toonDetails) do
		local toonName = toonValue.name;

		if (next(toonValue.profCooldowns) ~= nil) then	
			local professionCooldowns = toonValue.profCooldowns;

			reportText = EPiC:AppendLine(reportText, toonName .. "'s Cooldowns:");

			for key, value in pairs(professionCooldowns) do	
				reportText = EPiC:AppendLine(reportText, " -- " .. key .. " : " .. value.status);
			end

			--Separate toon cooldowns
			reportText = EPiC:AppendLine(reportText, "");

		else
			reportText = EPiC:AppendLine(reportText, toonName .. " has no cooldowns registered");
		end
	end

	--reportText = EPiC:AppendLine(reportText, "--------Detailed Report Completed--------");

	return reportName, reportText;
end

function EPiC:ToonDetails()
	return EPiC.db.realm.toonDetails[globalToonDetailKey];
end

function EPiC:RefreshConfig()
	self:Print("Refreshing Config")
end

function EPiC:GetToonDetailKey()
	local toonDetailKey = 0

	for i=0, EPiC.db.realm.toonsCount do
		if (EPiC.db.realm.toonDetails[i] ~= nil) then			
			if (EPiC.db.realm.toonDetails[i].name == charName) then
				toonDetailKey = i;
			end
		end
	end
	
	if (toonDetailKey == 0) then
		EPiC.db.realm.toonsCount = EPiC.db.realm.toonsCount + 1;
		toonDetailKey = EPiC.db.realm.toonsCount;
	end

	return toonDetailKey;
end

function EPiC:RegisterToon() 

	toonDetail.name = charName;
	toonDetail.professions.prof1, toonDetail.professions.prof2 = EPiC:GetMyProfessions();

	toonDetail.profCooldowns = EPiC:GetMyProfessionCooldowns();

	EPiC.db.realm.toonDetails[globalToonDetailKey] =  toonDetail;
	self:Print("Registered " .. EPiC:ToonDetails().name .. " under key " .. globalToonDetailKey);
end


function EPiC:OnProfessionOpen()
	local header = "OnProfessionOpen()";
	local numSkills = GetNumTradeSkills()

	local tradeskillName, rank, maxLevel = GetTradeSkillLine();
	
	local prof1Cooldowns = EPiC:GetStaticDailyCooldownsForProfession(EPiC:ToonDetails().professions.prof1);
	local prof2Cooldowns = EPiC:GetStaticDailyCooldownsForProfession(EPiC:ToonDetails().professions.prof2);

	local profCooldowns = EPiC:GetStaticDailyCooldownsForProfession(tradeskillName);
	EPiC:Debug(header, tradeskillName .. " lookup: " .. EPiC:SimpleArrayToString(profCooldowns));

	-- Expand tradeskill
	for i=1,GetNumTradeSkills() do
		local skillName, type = GetTradeSkillInfo(i);

		--If actual skill, check for matches otherwise expand headers
		if (skillName and type == "header" and type == "subheader") then
			ExpandTradeSkillSubClass(i);
		end
	end

	-- Get the registered skills of the same profession
	for key, profSkillName in ipairs(profCooldowns) do
		
		--Attempt to get the latest cooldown information
		local isMatch = false;
		
		local initCount = 1;
		local maxCount = 5;
		local attemptCount = initCount;

		while (not isMatch and attemptCount <= maxCount) do

			if (attemptCount > 1) then
				self:Print("Attempting to update " .. profSkillName .. ": " .. attemptCount .. " of " .. maxCount);
			end

			for i=1,GetNumTradeSkills() do
				local skillName, type = GetTradeSkillInfo(i);
				local cooldown = GetTradeSkillCooldown(i);

				if (profSkillName == skillName) then
					isMatch = true;
					self:Print("Updating " .. skillName);
					EPiC:UpdateProfessionCooldowns(tradeskillName, skillName, cooldown);
					break;
				end
			end

			attemptCount = attemptCount + 1;
		end
	end

	self:Print("Done with OnProfessionOpen");
end

function EPiC:IsMatchingSkill(profCooldowns, skillName)
	local header = "EPiC:IsMatchingSkill";
	local isMatch = false;

	for i, staticName in ipairs(profCooldowns) do

		if (skillName == staticName) then
			EPiC:Debug(header, "Found a match: " .. skillName .. " vs " .. staticName);
			isMatch = true;
			break;
		end
	end

	return isMatch;
end

function EPiC:GetMyProfessionCooldowns()
	local myProfessionCooldowns = {};

	return myProfessionCooldowns;
end


function EPiC:RefreshCooldowns(input)
	local logHeader = "RefreshCooldowns()";

	--For each registered toon
	for toonKey, toonValue in ipairs(EPiC.db.realm.toonDetails) do
		local toonName = toonValue.name;
		

		if (next(toonValue.profCooldowns) ~= nil) then	
			local professionCooldowns = toonValue.profCooldowns;

			for key, value in pairs(professionCooldowns) do

				if (not EPiC:IsOnCooldown(value.expirationDate)) then
					if (value.status ~= STATUS_READY) then
						EPiC:Log(toonName, key .. " is no longer on cooldown");
						EPiC.db.realm.toonDetails[toonKey].profCooldowns[key].status = STATUS_READY;
						EPiC.db.realm.toonDetails[toonKey].profCooldowns[key].expirationDate = EPiC:GetNewDateTime();
					end
				end
			end
		end
	end

	if (input ~= LOG_SUPPRESS) then
		EPiC:Log(logHeader, "Complete");
	end

	refreshCooldownsComplete=true;
end

function EPiC:UpdateProfessionCooldowns(profName, skillName, cooldown)

	local header = "EPiC:UpdateProfessionCooldowns";

	local cooldownHour = 0;
	local cooldownMin = 0;
	
	if (cooldown ~= nil) then
		cooldownHour, _, cooldownMin = strsplit(" ", SecondsToTime(cooldown));
	end

	local skillProfCooldown = {
		profession = uninitialized,
		status = uninitialized,
		expirationDate = {},
	};

	skillProfCooldown.profession = profName;

	local expirationDate = EPiC:GetExpirationDateTime(cooldownHour, cooldownMin);

	if (EPiC:IsOnCooldown(expirationDate)) then
		skillProfCooldown.expirationDate = expirationDate;
		skillProfCooldown.status = STATUS_ONCOOLDOWN;
	else
		skillProfCooldown.expirationDate = EPiC:GetNewDateTime();
		skillProfCooldown.status = STATUS_READY;
	end

	EPiC:Debug(header, skillProfCooldown.profession .. ":" .. skillName .. ":" .. EPiC:DateTimeToString(skillProfCooldown.expirationDate) .. ":" .. skillProfCooldown.status);

	EPiC:ToonDetails().profCooldowns[skillName] = skillProfCooldown;
end

function EPiC:GetMyProfessions()

	local tradeSkill1, tradeSkill2 = GetProfessions();

	if (tradeSkill1 == nil) then
		tradeSkill1 = unset;
	end

	if (tradeSkill2 == nil) then
		tradeSkill2 = unset;
	end

	return tradeSkill1, tradeSkill2;
end

function EPiC:GetStaticDailyCooldownsForProfession(prof)
	local logHeader = "EPiC:GetStaticDailyCooldownsForProfession";
	EPiC:Debug(logHeader, "Getting cooldowns");

	local staticCooldowns = {}
	local profName = uninitialized;

	if (type(prof) == "number") then
		EPiC:Debug(logHeader, "isNumber");
		profName = GetProfessionInfo(prof);
	elseif (type(prof) == "string") then
		EPiC:Debug(logHeader, "isString");
		profName = prof;
	else
		EPiC:Debug(logHeader, type(prof));
	end

	if		(profName == "Alchemy") then
		staticCooldowns = EPiC.db.global.staticCooldowns.Alchemy;
	elseif	(profName == "Blacksmithing") then
		staticCooldowns = EPiC.db.global.staticCooldowns.Blacksmithing;
	elseif	(profName == "Enchanting") then
		staticCooldowns = EPiC.db.global.staticCooldowns.Enchanting;
	elseif	(profName == "Engineering") then
		staticCooldowns = EPiC.db.global.staticCooldowns.Engineering;
	elseif	(profName == "Inscription") then
		staticCooldowns = EPiC.db.global.staticCooldowns.Inscription;
	elseif	(profName == "Jewelcrafting") then
		staticCooldowns = EPiC.db.global.staticCooldowns.Jewelcrafting;
	elseif	(profName == "Leatherworking") then
		staticCooldowns = EPiC.db.global.staticCooldowns.Leatherworking;
	elseif	(profName == "Tailoring") then
		staticCooldowns = EPiC.db.global.staticCooldowns.Tailoring;
	end

	return staticCooldowns;
end

function EPiC:GetProfessionFromSkillName(skillName)
	local logHeader = "EPiC:GetProfessionFromSkillName";

	for profName, profCooldowns in pairs(EPiC.db.global.staticCooldowns) do

		for i, profSkillName in ipairs(profCooldowns) do
			if(profSkillName == skillName) then
				return profName;
			end
		end
	end

	return "";
end

function EPiC:SimpleArrayToString(array)
	
	local initializeArrayString = '['

	local arrayString = initializeArrayString

	for i, value in ipairs(array) do

		if arrayString == initializeArrayString then
			arrayString = arrayString .. value
		else
			arrayString = arrayString .. ', ' .. value
		end

	end

	arrayString = arrayString .. ']'

	return arrayString
end


function EPiC:ReRegisterToon()
	EPiC:RegisterToon();
	ReloadUI();
end

function EPiC:EpicReload()
	ReloadUI();
end

function EPiC:UnitTest()
	local logHeader = "UnitTest";
	passedTests = 0;
	failedTests = 0;

	--Verify Refresh was successful
	EPiC:AssertEquals("Refresh Cooldowns", true, refreshCooldownsComplete);

	--True expiration
	local expirTime1 = EPiC:GetExpirationDateTime(2,0);
	EPiC:AssertEquals("Future Expiration 1", true, EPiC:IsOnCooldown(expirTime1));

	--True expiration
	local expirTime2 = EPiC:GetCurrentDateTime();
	expirTime2.day = expirTime2.day + 1;
	expirTime2.hour = 0;
	expirTime2.minute = 0;
	EPiC:AssertEquals("Future Expiration 2", true, EPiC:IsOnCooldown(expirTime2));

	--Same Dates
	EPiC:AssertEquals("Same DateTime", false, EPiC:IsOnCooldown(EPiC:GetCurrentDateTime()));

	--Expired expiration 1
	local pastExpiration1 = EPiC:GetNewDateTime();

	pastExpiration1.year = 2014;
	pastExpiration1.month = 12;
	pastExpiration1.day = 13;
	pastExpiration1.hour = 23;
	pastExpiration1.minute = 59;

	EPiC:AssertEquals("Past Expiration 1", false, EPiC:IsOnCooldown(pastExpiration1));

	--Expired expiration 2
	local pastExpiration2 = EPiC:GetNewDateTime();
	pastExpiration2.year = 2013;
	pastExpiration2.month = 12;
	pastExpiration2.day = 13;
	pastExpiration2.hour = 23;
	pastExpiration2.minute = 59;

	EPiC:AssertEquals("Past Expiration 2", false, EPiC:IsOnCooldown(pastExpiration2));

	--GetFilters 1 - cooldown
	local type, value = EPiC:GetFilters("cd");

	EPiC:AssertEquals("GetFilters 1 Type", FILTER_STATUS, type);
	EPiC:AssertEquals("GetFilters 1 Value", STATUS_ONCOOLDOWN, value);

	--GetFilters 2 - no value
	local type, value = EPiC:GetFilters("");

	EPiC:AssertEquals("GetFilters 2 Type", FILTER_NONE, type);
	EPiC:AssertEquals("GetFilters 2 Value", uninitialized, value);

	--GetFilters 3 - character
	local type, value = EPiC:GetFilters("Elezator");

	EPiC:AssertEquals("GetFilters 3 Type", FILTER_TOON, type);
	EPiC:AssertEquals("GetFilters 3 Value", "Elezator", value);

	--GetFilters 4 - ready
	local type, value = EPiC:GetFilters("ready");

	EPiC:AssertEquals("GetFilters 4 Type", FILTER_STATUS, type);
	EPiC:AssertEquals("GetFilters 4 Value", STATUS_READY, value);

	--Print Pass/Fail status
	EPiC:Log(logHeader, "Passed: " .. passedTests .. "; Failed: " .. failedTests);
end

function EPiC:AssertEquals(message, expected, actual)
	local logHeader = "AssertEquals";

	if		(type(expected) == "boolean") then 
		expectedString = EPiC:BooleanToString(expected); 
	else	
		expectedString = expected;
	end

	if		(type(actual) == "boolean") then 
		actualString = EPiC:BooleanToString(actual); 
	else	
		actualString = actual;
	end

	if (expected == actual) then
		EPiC:Log(logHeader, message .. " - " .. "Test Succeeded");
		passedTests = passedTests + 1;
	else
		EPiC:Log(logHeader, message .. " - " .. "Expected: " .. expectedString .. ", Actual: " .. actualString);
		failedTests = failedTests + 1;
	end
end

function EPiC:Log(header, message)
	self:Print(header .. " - " .. message);
end

function EPiC:Debug(header, message)
	if (debugging) then
		EPiC:Log(header, message);
	end
end

function EPiC:BooleanToString(booleanValue)
	local booleanString = "";
	
	if(booleanValue) then
		booleanString = "true";
	else
		booleanString = "false";
	end

	return booleanString;	
end

function EPiC:GetKeyFromValue(list, value)

	local key = 0;

	for k, v in ipairs(list) do
		if (value == v) then
			key = k;
		end
	end

	return key;
end

function EPiC:AppendLineWithPrefix(currentText, prefixText, messageText)
	return EPiC:AppendLine(currentText, prefixText .. " - " .. messageText);
end

function EPiC:AppendLine(currentText, messageText)
	local returnText = currentText .. messageText .. "\n";
	return returnText;
end